-- BetterScience for Factorio, by Taehl (email.Taehl@gmail.com)

local dat = data.raw
config = require"config"


if config.improveScienceRecipes then		-- new science 1-4 recipes and tech tree adjustment
	dat.recipe["science-pack-1"].ingredients = { {"copper-cable", 2}, {"iron-gear-wheel", 1}, }
	dat.recipe["science-pack-2"].ingredients = { {"science-pack-1", 1}, {"inserter", 1}, {"transport-belt", 1}, {"steel-plate", 1}, }
	dat.recipe["science-pack-3"].ingredients = { {"science-pack-2", 1}, {"filter-inserter", 1}, {"fast-transport-belt", 1}, {"solid-fuel", 1}, }
	dat.recipe["alien-science-pack"].ingredients = { {"science-pack-3", 1}, {"processing-unit", 1}, {"stack-inserter", 1}, {"express-transport-belt", 1}, }
	dat.recipe["alien-science-pack"].category = "crafting-with-fluid"
	dat.recipe["alien-science-pack"].order = "a[science-pack-4]"
	dat.recipe["alien-science-pack"].result_count = 1

	-- science 4 production can be doubled by adding artifacts
	data:extend( {
		{
			type = "recipe",
			name = "alien-science-pack-double",
			enabled = false,
			category = "crafting-with-fluid",
			order = "a[science-pack-4b]",
			energy_required = 12,
			ingredients = { {"alien-artifact", 1}, {"science-pack-3", 1}, {"processing-unit", 1}, {"stack-inserter", 1}, {"express-transport-belt", 1}, },
			result = "alien-science-pack",
			result_count = 2,
		},
	} )
	table.insert( dat.technology["alien-technology"].effects, {type = "unlock-recipe", recipe = "alien-science-pack-double"} )
	
	-- rebalance the tech tree for better progression and to ensure all ingredients are unlocked before needed in science production
	
	do	-- oil-processing (6 techs to research, total)
		-- science 3 is unlocked earlier, with oil-processing instead of batteries
		for k,v in pairs( dat.technology["battery"].effects ) do
			if v.recipe and v.recipe == "science-pack-3" then table.remove( dat.technology["battery"].effects, k ) end
		end
		table.insert( dat.technology["oil-processing"].effects, {type = "unlock-recipe", recipe = "science-pack-3"} )
		
		-- you need fast belts to build science 3, so you need to research logistics-2 before oil-processing
		table.insert( dat.technology["oil-processing"].prerequisites, "logistics-2" )
		
		-- science 3 requires filter inserters, so oil-processing should require electronics
		table.insert( dat.technology["oil-processing"].prerequisites, "electronics" )
	end
	
	do	-- advanced-oil-processing (14 techs total)
		-- science 4 unlocks a bit earlier with advanced-oil-processing instead of alien-technology
		table.insert( dat.technology["advanced-oil-processing"].effects, {type = "unlock-recipe", recipe = "alien-science-pack"} )
		for k,v in pairs( dat.technology["alien-technology"].effects ) do
			if v.recipe and v.recipe == "alien-science-pack" then table.remove( dat.technology["alien-technology"].effects, k ) end
		end
		
		-- science 4 now requires express belts, so advanced-oil-processing should require logistics-3
		table.insert( dat.technology["advanced-oil-processing"].prerequisites, "logistics-3" )
		
		-- science 4 requires stack inserters, but gets them indirectly via logistics-3 now requiring them
		table.insert( dat.technology["logistics-3"].prerequisites, "stack-inserter" )
		
		-- slightly delay science 4 by making advanced-oil-processing require sulphur processing (plus it's realistic)
		table.insert( dat.technology["advanced-oil-processing"].prerequisites, "sulfur-processing" )
	end
	
	do	-- alien-technology (20 techs)
		-- advanced science continues the science progression, so it should require advanced-oil-processing
		table.insert( dat.technology["alien-technology"].prerequisites, "advanced-oil-processing" )
		
		-- all recipes in alien-technology need assembling-machine-3, so add automation-3 to its prerequisites
		table.insert( dat.technology["alien-technology"].prerequisites, "automation-3" )
		
		-- should need productivity-module, to balance the research tree (and doubling science 4 output is "productivity")
		table.insert( dat.technology["alien-technology"].prerequisites, "productivity-module" )
		
		-- shouldn't require rocketry, since it's not military-related
		for k,v in ipairs( dat.technology["alien-technology"].prerequisites ) do
			if v == "rocketry" then table.remove( dat.technology["alien-technology"].prerequisites, k ) end
		end
		
		-- move advanced-electronics-2 from rocket-silo down to alien-technology
		table.insert( dat.technology["alien-technology"].prerequisites, "advanced-electronics-2" )
	
		-- needing only science 1-2, alien-technology feels a little like a step backwards cost-wise
		table.insert( dat.technology["alien-technology"].unit.ingredients, {"science-pack-3", 1} )
		table.insert( dat.technology["alien-technology"].unit.ingredients, {"alien-science-pack", 1} )
		dat.technology["alien-technology"].unit.count = 200
		
		-- these (and their dependants) need advanced science, so they should require alien-technology
		for k,dependant in ipairs{ "logistic-robotics", "construction-robotics", "military-4", } do
			table.insert( dat.technology[dependant].prerequisites, "alien-technology" )
		end
	end
	
	do	-- rocket-silo (31 techs, with explosives+flammables>rocketry being the only military research involved)
		-- building space rockets shouldn't require the ability to rapidly fire a rocket-launcher (IMHO, leftover from the old Rocket Defense days)
		for k,v in ipairs( dat.technology["rocket-silo"].prerequisites ) do
			if v == "rocket-speed-5" then table.remove( dat.technology["rocket-silo"].prerequisites, k ) end
		end
		
		-- but now it needs to require alien-technology directly
		table.insert( dat.technology["rocket-silo"].prerequisites, "alien-technology" )
		
		-- no longer needs advanced-electronics-2 because it's in alien-technology
		for k,v in ipairs( dat.technology["rocket-silo"].prerequisites ) do
			if v == "advanced-electronics-2" then table.remove( dat.technology["rocket-silo"].prerequisites, k ) end
		end
		
		-- should require battery because, really, you want to build a rocket which can't store electricity? (also, the satellite needs accumulators)
		table.insert( dat.technology["rocket-silo"].prerequisites, "battery" )
		
		-- should need effectivity-module-3, to balance out the tree (and nobody wants to make ineffective rockets)
		table.insert( dat.technology["rocket-silo"].prerequisites, "effectivity-module-3" )
	end
	
	do	-- logistics-3 needs minor adjustment
		-- to make logistics-3 more consistent and parallel, move it from requiring automation-3 to effectivity-module
		for k,v in ipairs( dat.technology["logistics-3"].prerequisites ) do
			if v == "automation-3" then dat.technology["logistics-3"].prerequisites[k] = "effectivity-module" end
		end
		
		-- logistics-3 takes more science than advanced-oil-processing, which doesn't flow evenly
		dat.technology["logistics-3"].unit.count = 150
		dat.technology["advanced-oil-processing"].unit.count = 200
	end
	
	-- combat-robotics-3 should require military-4, to continue mutual progression and inherit need for advanced science
	table.insert( dat.technology["combat-robotics-3"].prerequisites, "military-4" )
	
end


if config.enableTransformerOil then		-- transformer oil added to science 4 recipe
	data:extend( {
		{
			type = "fluid",
			name = "transformer-oil",
			default_temperature = 50,
			heat_capacity = "2KJ",
			base_color = {r=.7, g=.6, b=0},
			flow_color = {r=.85, g=.9, b=.2},
			max_temperature = 200,
			icon = "__BetterScience__/graphics/transformer-oil.png",
			pressure_to_speed_ratio = 0.4,
			flow_to_energy_ratio = 0.59,
			order = "a[fluid]-b[transformer-oil]"
		},
		{
			type = "recipe",
			name = "transformer-oil-processing",
			enabled = false,
			category = "chemistry",
			energy_required = 5,
			ingredients = { {type="fluid", name="petroleum-gas", amount=1}, {type="fluid", name="water", amount=4}, },
			results = { {type="fluid", name="transformer-oil", amount=1} },
			icon = "__BetterScience__/graphics/make-transformer-oil.png",
			subgroup = "fluid-recipes",
			order = "b[fluid-chemistry]-bb[transformer-oil]"
		},
	} )
	
	-- add to science 4 recipes
	table.insert( dat.recipe["alien-science-pack"].ingredients, {type="fluid", name="transformer-oil", amount=0.2} )
	if config.improveScienceRecipes then
		table.insert( dat.recipe["alien-science-pack-double"].ingredients, {type="fluid", name="transformer-oil", amount=0.4} )
	end
	
	-- unlocked with advanced oil processing
	table.insert( dat.technology["advanced-oil-processing"].effects, {type = "unlock-recipe", recipe = "transformer-oil-processing"} )
end


local groups
if config.enableScience5 then		-- add advanced science packs to complicate end-game research
	-- lab needs to have more slots
	dat.lab.lab.inputs = { "science-pack-1", "science-pack-2", "science-pack-3", "alien-science-pack", "science-pack-5-logistics", "science-pack-5-production", "science-pack-5-robotics", "science-pack-5-military" }

	-- new science packs
	local packs = { "logistics", "production", "robotics", "military" }
	local recipes = {
		{ {"alien-science-pack", 1}, {"speed-module", 3}, {"stack-filter-inserter", 1}, {"electric-engine-unit", 1}, {"red-wire", 1}, {"green-wire", 1}, },
		{ {"alien-science-pack", 1}, {"productivity-module", 2}, {"assembling-machine-2", 1}, {"chemical-plant", 1}, },
		{ {"alien-science-pack", 1}, {"effectivity-module", 2}, {"construction-robot", 1}, {"advanced-circuit", 1}, {"accumulator", 4}, },
		{ {"alien-science-pack", 1}, {"energy-shield-equipment", 1}, {"distractor-capsule", 1}, {"rocket", 2}, {"radar", 2}, },
	}
	for k, packType in ipairs( packs ) do
		local name = "science-pack-5-"..packType
		data:extend( {
			{
				type = "tool",
				name = name,
				icon = "__BetterScience__/graphics/"..name..".png",
				flags = {"goes-to-main-inventory"},
				subgroup = "science-pack",
				order = "a["..name.."]",
				stack_size = 200,
				durability = 1,
				durability_description_key = "description.science-pack-remaining-amount"
			},
		{
			type = "recipe",
			name = name,
			enabled = false,
			energy_required = k==2 and 40 or 10,		-- production science takes much more energy
			ingredients = recipes[k],
			result = name,
			result_count = 1,
		},
		} )
		
		-- unlocked upon researching alien-technology
		table.insert( dat.technology["alien-technology"].effects, {type = "unlock-recipe", recipe = name} )
	end
	
	-- group end-game research into new science categories (listed numbers are which level to start adding advanced science)
	groups = {
		[0] = packs,
		{"logistics-4", "speed-module-3", "inserter-capacity-bonus-6", "character-logistic-slots-5", "logistic-robotics", "worker-robots-storage-3", },
		{"automation-4", "productivity-module-3", "rocket-silo", "power-armor-2", "construction-robotics", "automated-construction", "research-speed-4", "advanced-electronics-3"},
		{"robotics-4", "effectivity-module-3", "worker-robots-speed-4", "combat-robotics-3", "combat-robot-damage-5", "follower-robot-count-8", },
		{"military-4", "cluster-grenade", "grenade-damage-5", "shotgun-shell-damage-6", "shotgun-shell-speed-6", "laser-turret-damage-6", "laser-turret-speed-6", "gun-turret-damage-6", "flamethrower-damage-5", "bullet-damage-6", "bullet-speed-6", "rocket-damage-5", "rocket-speed-5", },
	}, {}
	-- now change that list into the form of: groups["speed-module"] = { 3, "logistics" }
	for i = 1,4 do
		for k,item in ipairs( groups[i] ) do 
			local name, level = string.match( item, "([%w%p]+)%-(%d+)" )
			name = name or string.match( item, "([%w%p]+)" )		-- if it doesn't have a level, like "logistic-robots"
			groups[name] = { tonumber( level or 0 ), groups[0][i] }
		end
	end
end


-- rebalance research costs to compensate for more higher-tier science being more expensive
for k,tech in pairs( dat.technology ) do
	-- configurable costs multiplier
	local count = tech.unit.count * (config.scienceCostMultiplier or 1)
	tech.unit.count = count
	
	-- find the highest level of research required (if there's not a number in the name, it's "alien-science-pack" 4)
	local highest = 0
	for k,ingredient in pairs( tech.unit.ingredients ) do
		local n = tonumber( string.match(ingredient[1], "(%d)") or 4 )
		if n > highest then highest = n end
	end
	
	if config.enableScience5 then		-- check if new science needs to be added (if it's in the specified groups)
		local techName, techLevel = string.match( tech.name, "([%w%p]+)-(%d+)" )
		techName, techLevel = techName or string.match( tech.name, "([%w%p]+)" ), tonumber( techLevel or 0 )	-- if it doesn't have a level
		if groups[techName] then
			if groups[techName][1] <= techLevel then
				highest = 5
				table.insert( tech.unit.ingredients, { "science-pack-5-"..groups[techName][2], 1} )
			end
		end
	end
	
	if config.rebalanceTechnologyCosts then		-- new base cost of sciences 1-4
		local function nearest(n) return math.floor(n/5+.5)*5 end
		local newCosts
		if highest < 2 then newCosts = { 1 }		-- recipes which only have science 1 don't need adjusting
		elseif highest == 2 then count, newCosts = nearest( count /2 ), { 2, 1 }		-- just to start the pattern
		elseif highest == 3 then count, newCosts = nearest( count /2 ), { 2, 2, 1 }
		elseif highest == 4 then count, newCosts = nearest( count /2 /2 ), { 2,2,2, 1 }
		elseif highest >= 5 then count, newCosts = nearest( count /4 /2 /2 ), { 4,4,4, 2, 1 }
		end
		
		-- update recipe ingredients
		tech.unit.count = count
		for k,ingredient in pairs( tech.unit.ingredients ) do
			local n = tonumber( string.match(ingredient[1], "(%d)") or 4 )
			ingredient[2] = newCosts[n] * ingredient[2]		-- multiplied by itself because recipes often demand more than 1x
		end
	end
end